@@ -89,7 +89,7 @@ class HTTPPasswordHandler(object):
     def __init__(self):
         self.pwd_cache = {}
         self.last_reply = None
-        
+
     def find_auth(self, pwmgr, realm, authuri):
         """
         Actual implementation of find_user_password - different
@@ -104,7 +104,7 @@ class HTTPPasswordHandler(object):
         after_bad_auth = (self.last_reply \
                           and (self.last_reply['realm'] == realm) \
                           and (self.last_reply['authuri'] == authuri))
-           
+
         # Strip arguments to get actual remote repository url.
         base_url = self.canonical_url(authuri)
 
@@ -113,12 +113,12 @@ class HTTPPasswordHandler(object):
         user, pwd = urllib2.HTTPPasswordMgrWithDefaultRealm.find_user_password(
             pwmgr, realm, authuri)
         if user and pwd:
-            self._debug_reply(ui, _("Auth data found in repository URL"), 
+            self._debug_reply(ui, _("Auth data found in repository URL"),
                               base_url, user, pwd)
             self.last_reply = dict(realm=realm,authuri=authuri,user=user)
             return user, pwd
 
-        # Loading .hg/hgrc [auth] section contents. If prefix is given, 
+        # Loading .hg/hgrc [auth] section contents. If prefix is given,
         # it will be used as a key to lookup password in the keyring.
         auth_user, pwd, prefix_url = self.load_hgrc_auth(ui, base_url)
         if prefix_url:
@@ -133,7 +133,7 @@ class HTTPPasswordHandler(object):
             cached_auth = self.pwd_cache.get(cache_key)
             if cached_auth:
                 user, pwd = cached_auth
-                self._debug_reply(ui, _("Cached auth data found"), 
+                self._debug_reply(ui, _("Cached auth data found"),
                                   base_url, user, pwd)
                 self.last_reply = dict(realm=realm,authuri=authuri,user=user)
                 return user, pwd
@@ -144,25 +144,25 @@ class HTTPPasswordHandler(object):
             user = auth_user
             if pwd:
                 self.pwd_cache[cache_key] = user, pwd
-                self._debug_reply(ui, _("Auth data set in .hg/hgrc"), 
+                self._debug_reply(ui, _("Auth data set in .hg/hgrc"),
                                   base_url, user, pwd)
                 self.last_reply = dict(realm=realm,authuri=authuri,user=user)
                 return user, pwd
             else:
                 ui.debug(_("Username found in .hg/hgrc: %s\n" % user))
 
-        # Loading password from keyring. 
+        # Loading password from keyring.
         # Only if username is known (so we know the key) and we are
         # not after failure (so we don't reuse the bad password).
         if user and not after_bad_auth:
             pwd = password_store.get_http_password(keyring_url, user)
             if pwd:
                 self.pwd_cache[cache_key] = user, pwd
-                self._debug_reply(ui, _("Keyring password found"), 
+                self._debug_reply(ui, _("Keyring password found"),
                                   base_url, user, pwd)
                 self.last_reply = dict(realm=realm,authuri=authuri,user=user)
                 return user, pwd
-        
+
         # Is the username permanently set?
         fixed_user = (user and True or False)
 
@@ -172,7 +172,7 @@ class HTTPPasswordHandler(object):
 
         if not fixed_user:
             ui.status(_("Username not specified in .hg/hgrc. Keyring will not be used.\n"))
-            
+
         ui.write(_("http authorization required\n"))
         ui.status(_("realm: %s\n") % realm)
         if fixed_user:
@@ -192,7 +192,7 @@ class HTTPPasswordHandler(object):
         # Saving password to the memory cache
         self.pwd_cache[cache_key] = user, pwd
 
-        self._debug_reply(ui, _("Manually entered password"), 
+        self._debug_reply(ui, _("Manually entered password"),
                           base_url, user, pwd)
         self.last_reply = dict(realm=realm,authuri=authuri,user=user)
         return user, pwd
@@ -205,23 +205,23 @@ class HTTPPasswordHandler(object):
         element can be None)
         """
         # Theoretically 3 lines below should do:
-        
+
         #auth_token = self.readauthtoken(base_url)
         #if auth_token:
         #   user, pwd = auth.get('username'), auth.get('password')
-        
+
         # Unfortunately they do not work, readauthtoken always return
         # None. Why? Because ui (self.ui of passwordmgr) describes the
         # *remote* repository, so does *not* contain any option from
         # local .hg/hgrc.
 
         # TODO: mercurial 1.4.2 is claimed to resolve this problem
-        # (thanks to: http://hg.xavamedia.nl/mercurial/crew/rev/fb45c1e4396f) 
+        # (thanks to: http://hg.xavamedia.nl/mercurial/crew/rev/fb45c1e4396f)
         # so since this version workaround implemented below should
         # not be necessary. As it will take some time until people
         # migrate to >= 1.4.2, it would be best to implement
         # workaround conditionally.
-        
+
         # Workaround: we recreate the repository object
         repo_root = ui.config("bundle", "mainreporoot")
 
@@ -229,8 +229,17 @@ class HTTPPasswordHandler(object):
         local_ui = _ui(ui)
         if repo_root:
             local_ui.readconfig(os.path.join(repo_root, ".hg", "hgrc"))
-        local_passwordmgr = passwordmgr(local_ui)
-        auth_token = local_passwordmgr.readauthtoken(base_url)
+        try:
+            local_passwordmgr = passwordmgr(local_ui)
+            auth_token = local_passwordmgr.readauthtoken(base_url)
+        except AttributeError:
+            # hg 1.8
+            from mercurial.url import readauthforuri
+            res = readauthforuri(local_ui, base_url)
+            if res:
+                group, auth_token = res
+            else:
+                auth_token = None
         if auth_token:
             username = auth_token.get('username')
             password = auth_token.get('password')
@@ -289,7 +298,7 @@ def find_user_password(self, realm, authuri):
 def try_smtp_login(ui, smtp_obj, username, password):
     """
     Attempts smtp login on smtp_obj (smtplib.SMTP) using username and
-    password. 
+    password.
 
     Returns:
     - True if login succeeded
@@ -340,7 +349,7 @@ def keyring_supported_smtp(ui, username):
         s.ehlo()
         s.starttls()
         s.ehlo()
-        
+
     #>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     stored = password = password_store.get_smtp_password(
         mailhost, mailport, username)
